# $Id: sm_notify.ps1,v 1.5 2012/06/24 14:02:57 seniorix Exp $
# ******************************************************************************
#
#  File      : sm_notify.ps1
#  Copyright : (c) SEP AG  D-83607 Holzkirchen
#  Project   : sesam Backup Utility
#  Date      : 2017-05-16
#
# ******************************************************************************
#
#  Description : This is the notify interface of sesam backup utility on MS Windows.
#                It is called from sesam during finishing a backup, media change,
#                media init, sesam day change, all scheduled jobs finished
#                or sesam startup.
#
#  Parameters  : $args[0] = Calling module
#                $args[1] = Text of message
#
#  Modules     : ALL_JOBS_DONE
#                  Will be triggered after the last planned Sesam event of the Sesam day
#                  has been finished. Serves as KeepAlive report and Sesam day summary.
#                INIT
#                  A initialize (preparation) of a medium for backup has been finished succesfully.
#                GET_VOLUME
#                  A medium has been requested for backup and the operation has been
#                  finished successfully.
#                BACKUP
#                  A backup process has been finished successfully.
#                BACKUP_GROUP
#                  After the last backup task of a taskgroup has been finished successfully.
#                RESTORE
#                  A restore process has been finished successfully.
#                COPY
#                  A migration operation has been finished successfully.
#                  A migration operation contains one or more saveset migration processes.
#                COPY_SAVESET
#                  A migration task process has been finished successfully.
#                  This event will be triggerd on migration of each saveset
#                  in a migration oparation.
#                NEWDAY
#                  A Sesam NewDay event has been started.
#                SM_NEWDAY
#                  A Sesam NewDay event has been finished.
#
#  This is a user programable Powershell-Script-file with 2 parameters.
#  Please insert your specific code-lines after this comment.
#  Take care of not inserting commands which wait for an answer from
#  the user ( OK-buttons etc. ) - this may cause a total blocking of
#  backups during the night.
#
# ********************************************************************************
#
#  Beschreibung: Die Notify-Schnittstelle der sesam Sicherungssoftware unter MS-Windows.
#                Sie wird beim Beenden einer Sicherung, Medienwechsel, Medieninit,
#                sesam Tageswechsel, alle geplanten Sicherung beendet
#                oder sesam Start aufgerufen.
#
#  Parameter   : $args[0] = Name des Moduls
#                $args[1] = Text der Meldung
#
#  Module      : ALL_JOBS_DONE
#                  Wird ausgeloest nach Beendigung des letzten geplanten Events des
#                  Sesam-Tages. Dient als KeepAlive-Meldung und Tageszusammenfassung.
#                INIT
#                  Das Initialisieren (Bereitstellen) eines Mediums wurde erfolgreich beendet.
#                GET_VOLUME
#                  Ein Medium wird wird zum Backup angefordert (z.B. Einlegen des Bandes
#                  in ein Bandlaufwerk).
#                BACKUP
#                  Ein Backup-Prozess wurde erfolgreich beendet.
#                BACKUP_GROUP
#                  Nachdem der letzte Backup-Job einer Auftragsgruppe beendet wurde.
#                RESTORE
#                  Ein Restore-Prozess wurde erfolgreich beendet.
#                COPY
#                  Eine Migrations-Operation wurde erfolgreich beendet.
#                  Eine Migrations-Operation besteht aus einer oder mehreren Saveset Migrationen.
#                COPY_SAVESET
#                  Die Migration eines Savesets wurde erfolgreich abgeschlossen.
#                  Dieses Event wird bei jedem Saveset einer Migrations-Operation ausgeloest.
#                NEWDAY
#                  Ein Sesam Tageswechsel wurde gestartet.
#                SM_NEWDAY
#                  Ein Sesam Tageswechsel wurde beendet.
#
#  Dies ist eine benutzerprogrammierbare Powershell-Script-Datei mit 2 Parametern.
#  Fuegen Sie bitte Ihre eigenen Kommandozeilen nach diesem Kommentar ein,
#  wobei Kommandos, die eine Antwort erwarten ( OK-Buttons etc. ) vermieden
#  werden sollten, da sie Sicherungen waehrend der Nacht blockieren koennten.
#
#
# ********************************************************************************

if (Test-Path "HKLM:\SOFTWARE\SEP Elektronik GmbH")
{
   $ini = ((get-itemproperty 'HKLM:\SOFTWARE\SEP Elektronik GmbH\sesam\GLBV\gv_rw_ini')."(default)").Replace("/","\")
} else {
   $ini = ((get-itemproperty 'HKLM:\SOFTWARE\WOW6432Node\SEP Elektronik GmbH\sesam\GLBV\gv_rw_ini')."(default)").Replace("/","\")
}

# Execute SEP sesam profile
. $ini\sm_prof.ps1

# Get language string
$lang = (Get-Content -Path $ini\sm.ini | Where-Object { $_ -match 'lang=' }).Split('=')[1]

# Definition of mail send functions
function mail_attach($subject,$mail_user)
{
   sm_smtp -A "$mail_user" -s "$subject" -M "`"$env:gv_dayfile`"" -a "`"$env:gv_prot`"";
}

function mail_send($sub,$mess,$mail_user)
{
   sm_smtp -A "$mail_user" -s "$sub" -m "$mess";
}

# Print statusfile, different possibilities
function print_attach()
{
   # Print sesam dayfile to default printer on this system
   Out-Printer -InputObject (get-content $env:gv_dayfile)

   # Print sesam dayilfe to a defined printer, which is not default printer on this system (MyPrinter is the name of a printer on this computer)
   # Out-Printer -InputObject (get-content $env:gv_dayfile) -Name "MyPrinter"
}

# Eject tape from drive
function unload_tape($drive)
{
   sm_drive dismount $drive unload
}

# Load tape back from drive into slot
function load_to_slot($loader,$drive)
{
   sm_loader unload -l $loader -d $drive
}

# function for reporting backup state last sesam day
function report_backups_last_sesam_day($language,$mail_user)
{
   # investigate yesterday date
   $yesterday="$(sm_glbv r gv_yesterday)"
   # adding time part for compatibility
   $yesterday_time="${yesterday} 00:00:00"
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Sicherungsstatus des Sesam Tages $yesterday"
   } else {
      $subject="SEP sesam - Backup report sesam day $yesterday"
   }
   # report backup state last sesam day
   sm_cmd start report overview-backups -B "$yesterday_time" -E "$yesterday_time" -D "sesam_date" -l "$language" | Out-File -FilePath "${gv_rw_tmp}\backups_last_sesam_day.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M ${gv_rw_tmp}\backups_last_sesam_day.html -a "`"${gv_rw_tmp}\backups_last_sesam_day.html`""
}

# function for reporting backup state last sesam date for a specified location
function report_backups_last_sesam_day_location($language,$location,$mail_user)
{
   # investigate yesterday date
   $yesterday="$(sm_glbv r gv_yesterday)"
   # adding time part for compatibility
   $yesterday_time="${yesterday} 00:00:00"
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Sicherungsstatus des Sesam Tages $yesterday f�r Standort $location"
   } else {
      $subject="SEP sesam - Backup report sesam day $yesterday for location $location"
   }
   # report backup state last sesam day for specified location
   sm_cmd start report overview-backups -B "$yesterday_time" -E "$yesterday_time" -D "sesam_date" -l "$language" -p "{\\\""location_id\\\"":\\\""$location\\\""}" | Out-File -FilePath "${gv_rw_tmp}\backups_last_sesam_day_location_$location.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M "`"${gv_rw_tmp}\backups_last_sesam_day_location_$location.html`"" -a "`"${gv_rw_tmp}\backups_last_sesam_day_location_$location.html`""
}

# function for reporting backup state last 24 hours from execution time
function report_backups_last_24h($language,$mail_user)
{
   # investigate date and time for last 24 hours
   # day_time is date and time of today
   $day_time=(get-date -UFormat "%Y-%m-%d %H:%M:%S")
   # yesterday_time is the date and time in the past
   $yesterday_time=(get-date (get-date).addDays(-1) -UFormat "%Y-%m-%d %H:%M:%S")
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Sicherungsstatus der letzten 24 Stunden vom $yesterday_time"
   } else {
      $subject="SEP sesam - Backup report last 24 hours from $yesterday_time"
   }
   # report backup state last 24 hours
   sm_cmd start report overview-backups -B "$yesterday_time" -E "$day_time" -D "start_time" -l "$language" | Out-File -FilePath "${gv_rw_tmp}/backups_last_24h.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M "`"${gv_rw_tmp}\backups_last_24h.html`"" -a "`"${gv_rw_tmp}\backups_last_24h.html`""
}

# function for reporting backup state last 24 hours for a specified location
function report_backups_last_24h_location($language,$location,$mail_user)
{
   # investigate date and time for last 24 hours
   # day_time is date and time of today
   $day_time=(get-date -UFormat "%Y-%m-%d %H:%M:%S")
   # yesterday_time is the date and time in the past
   $yesterday_time=(get-date (get-date).addDays(-1) -UFormat "%Y-%m-%d %H:%M:%S")
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Sicherungsstatus der letzten 24 Stunden von $day_time f�r Standort $location"
   } else {
      $subject="SEP sesam - Backup report last 24 hours from $yesterday_time for location $location"
   }
   # report backup state last 24 hours for specified location
   sm_cmd start report overview-backups -B "$yesterday_time" -E "$day_time" -D "start_time" -l "$language" -p "{\\\""location_id\\\"":\\\""$location\\\""}" | Out-File -FilePath "${gv_rw_tmp}\backups_last_24h_location_$location.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M "`"${gv_rw_tmp}\backups_last_24h_location_$location.html`"" -a "`"${gv_rw_tmp}\backups_last_24h_location_$location.html`""
}

# function for reporting backup storage location last 24 hours from execution time
function report_backup_storage_last_24h($language,$mail_user)
{
   # investigate date and time for last 24 hours
   # day_time is date and time of today
   $day_time=(get-date -UFormat "%Y-%m-%d %H:%M:%S")
   # yesterday_time is the date and time in the past
   $yesterday_time=(get-date (get-date).addDays(-1) -UFormat "%Y-%m-%d %H:%M:%S")
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Speicher�bersicht der Sicherungen der letzten 24 Stunden vom $yesterday_time"
   } else {
      $subject="SEP sesam - Backup storage report last 24 hours from $yesterday_time"
   }
   # report backup storage location last 24 hours
   sm_cmd start report backup-storage-report -B "$yesterday_time" -E "$day_time" -D "start_time" -l "$language" | Out-File -FilePath "${gv_rw_tmp}/backup_storage_report_last_24h.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M "`"${gv_rw_tmp}\backup_storage_report_last_24h.html`"" -a "`"${gv_rw_tmp}\backup_storage_report_last_24h.html`""
}

# function for reporting backup storage location last 24 hours for a specified location
function report_backup_storage_last_24h_location($language,$location,$mail_user)
{
   # investigate date and time for last 24 hours
   # day_time is date and time of today
   $day_time=(get-date -UFormat "%Y-%m-%d %H:%M:%S")
   # yesterday_time is the date and time in the past
   $yesterday_time=(get-date (get-date).addDays(-1) -UFormat "%Y-%m-%d %H:%M:%S")
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Speicher�bersicht der Sicherungen der letzten 24 Stunden von $day_time f�r Standort $location"
   } else {
      $subject="SEP sesam - Backup storage report last 24 hours from $yesterday_time for location $location"
   }
   # report backup storage location last 24 hours for specified location
   sm_cmd start report backup-storage-report -B "$yesterday_time" -E "$day_time" -D "start_time" -l "$language" -p "{\\\""location_id\\\"":\\\""$location\\\""}" | Out-File -FilePath "${gv_rw_tmp}\backup_storage_report_last_24h_location_$location.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M "`"${gv_rw_tmp}\backup_storage_report_last_24h_location_$location.html`"" -a "`"${gv_rw_tmp}\backup_storage_report_last_24h_location_$location.html`""
}

# function for reporting failed jobs last 24 hours
function report_failed_jobs_last_24h($language,$mail_user)
{
   # investigate date and time for last 24 hours
   # day_time is date and time of today
   $day_time=(get-date -UFormat "%Y-%m-%d %H:%M:%S")
   # yesterday_time is the date and time in the past
   $yesterday_time=(get-date (get-date).addDays(-1) -UFormat "%Y-%m-%d %H:%M:%S")
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Fehlgeschlagene Sicherungen der letzten 24 Stunden vom $yesterday_time"
   } else {
      $subject="SEP sesam - Failed backup report last 24 hours from $yesterday_time"
   }
   # report of failed jobs last 24 hours
   sm_cmd start report failed-jobs-report -B "$yesterday_time" -E "$day_time" -D "start_time" -l "$language" | Out-File -FilePath "${gv_rw_tmp}\failed_jobs_last_24h.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M "`"${gv_rw_tmp}\failed_jobs_last_24h.html`"" -a "`"${gv_rw_tmp}\failed_jobs_last_24h.html`""
}

# function for reporting used media last 24 hours
function report_used_media_last_24h($language,$mail_user)
{
   # investigate date and time for last 24 hours
   # day_time is date and time of today
   $day_time=(get-date -UFormat "%Y-%m-%d %H:%M:%S")
   # yesterday_time is the date and time in the past
   $yesterday_time=(get-date (get-date).addDays(-1) -UFormat "%Y-%m-%d %H:%M:%S")
   # define different subject lines regarding to language
   if ( "$language" -eq "de" ) {
      $subject="SEP sesam - Benutze Medien der letzten 24 Stunden vom $yesterday_time"
   } else {
      $subject="SEP sesam - Used media report last 24 hours from $yesterday_time"
   }
   # report of used media last 24 hours
   sm_cmd start report used-media-report -B "$yesterday_time" -E "$day_time" -D "start_time" -l "$language" | Out-File -FilePath "${gv_rw_tmp}\used_media_last_24h.html" -Encoding "ascii"
   sm_smtp -A "$mail_user" -s "$subject" -M "`"${gv_rw_tmp}\used_media_last_24h.html`"" -a "`"${gv_rw_tmp}\used_media_last_24h.html`""
}

# Definition of variables
$command = $args[0]
$message = $args[1]
$task = ($args[1] -replace ":.*","")
$mail_subject = "Sesam: $command of $task"
$gv_rw_tmp=$env:tmp
$mail_user="sesam"
$location="0"

# Decision of what to do

# In Grolar by default "mail_attach" was activated in ALL_JOBS_DONE
# Since Beefalo by default "report_backups_last_24_hours" and "report_backup_storage_last_24_hours" is activated in NEWDAY
switch($command){
   ALL_JOBS_DONE {
      # mail_attach "Sesam backup status: $message" $mail_user;
      # print_attach;
      # load_to_slot 1 2;
      # unload_tape 2
      #
      # WH: eject only if a tape was initialized
      if ((Get-Content $env:gv_dayfile | select-string -Pattern "INIT" | Out-String) -ne "" ){
        Write-Host "WH: INIT found, eject tape." -fore green;
        # WH: eject tape
        unload_tape 2;
        # WH: print tape label
        Write-Host "WH: starting wh_print.ps1." -fore green;
        & ($env:sesam + "wh_print.ps1");
      } else {
        Write-Host "WH: INIT wasn't found, do not eject tape, do not create label." -fore red;
      }
      #
      break
   }
   GET_VOLUME {
      # mail_send $mail_subject $message "$mail_user";
      break
   }
   INIT {
      # mail_send $mail_subject $message "$mail_user";
      break
   }
   BACKUP {
      # mail_send $mail_subject $message "$mail_user"
      break
   }
   BACKUP_GROUP: {
      # mail_send $mail_subject $message "$mail_user"
      break
   }
   RESTORE {
      # mail_send $mail_subject $message "$mail_user"
      break
   }
   COPY {
      # mail_send $mail_subject $message "$mail_user"
      break
   }
   COPY_SAVESET {
      # mail_send $mail_subject $message "$mail_user"
      break
   }
   NEWDAY {
      # mail_send $mail_subject $message "$mail_user"
      # report_backups_last_sesam_day "$lang" "$mail_user";
      # report_backups_last_sesam_day_location "$lang" "$location" "$mail_user";
      report_backups_last_24h "$lang" "$mail_user";
      # report_backups_last_24h_location "$lang" "$location" "$mail_user";
      report_backup_storage_last_24h "$lang" "$mail_user";
      # report_backup_storage_last_24h_location "$lang" "$location" "$mail_user";
      report_failed_jobs_last_24h "$lang" "$mail_user";
      # report_used_media_last_24h "$lang" "$mail_user";
      break
   }
   SM_NEWDAY {
      # mail_send $mail_subject $message "$mail_user"
      break
   }
   default {
      # mail_send $mail_subject $message "$mail_user"
      break
   }
}

# SIG # Begin signature block
# MIIdggYJKoZIhvcNAQcCoIIdczCCHW8CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUk8C2CCdcky9Myq7KqhYOU2GA
# LcigghiTMIIEhDCCA2ygAwIBAgIQQhrylAmEGR9SCkvGJCanSzANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJTRTEUMBIGA1UEChMLQWRkVHJ1c3QgQUIxJjAkBgNV
# BAsTHUFkZFRydXN0IEV4dGVybmFsIFRUUCBOZXR3b3JrMSIwIAYDVQQDExlBZGRU
# cnVzdCBFeHRlcm5hbCBDQSBSb290MB4XDTA1MDYwNzA4MDkxMFoXDTIwMDUzMDEw
# NDgzOFowgZUxCzAJBgNVBAYTAlVTMQswCQYDVQQIEwJVVDEXMBUGA1UEBxMOU2Fs
# dCBMYWtlIENpdHkxHjAcBgNVBAoTFVRoZSBVU0VSVFJVU1QgTmV0d29yazEhMB8G
# A1UECxMYaHR0cDovL3d3dy51c2VydHJ1c3QuY29tMR0wGwYDVQQDExRVVE4tVVNF
# UkZpcnN0LU9iamVjdDCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAM6q
# gT+jo2F4qjEAVZURnicPHxzfOpuCaDDASmEd8S8O+r5596Uj71VRloTN2+O5bj4x
# 2AogZ8f02b+U60cEPgLOKqJdhwQJ9jCdGIqXsqoc/EHSoTbL+z2RuufZcDX65OeQ
# w5ujm9M89RKZd7G3CeBo5hy485RjiGpq/gt2yb70IuRnuasaXnfBhQfdDWy/7gbH
# d2pBnqcP1/vulBe3/IW+pKvEHDHd17bR5PDv3xaPslKT16HUiaEHLr/hARJCHhrh
# 2JU022R5KP+6LhHC5ehbkkj7RwvCbNqtMoNB86XlQXD9ZZBt+vpRxPm9lisZBCzT
# bafc8H9vg2XiaquHhnUCAwEAAaOB9DCB8TAfBgNVHSMEGDAWgBStvZh6NLQm9/rE
# JlTvA73gJMtUGjAdBgNVHQ4EFgQU2u1kdBScFDyr3ZmpvVsoTYs8ydgwDgYDVR0P
# AQH/BAQDAgEGMA8GA1UdEwEB/wQFMAMBAf8wEQYDVR0gBAowCDAGBgRVHSAAMEQG
# A1UdHwQ9MDswOaA3oDWGM2h0dHA6Ly9jcmwudXNlcnRydXN0LmNvbS9BZGRUcnVz
# dEV4dGVybmFsQ0FSb290LmNybDA1BggrBgEFBQcBAQQpMCcwJQYIKwYBBQUHMAGG
# GWh0dHA6Ly9vY3NwLnVzZXJ0cnVzdC5jb20wDQYJKoZIhvcNAQEFBQADggEBAE1C
# L6bBiusHgJBYRoz4GTlmKjxaLG3P1NmHVY15CxKIe0CP1cf4S41VFmOtt1fcOyu9
# 08FPHgOHS0Sb4+JARSbzJkkraoTxVHrUQtr802q7Zn7Knurpu9wHx8OSToM8gUmf
# ktUyCepJLqERcZo20sVOaLbLDhslFq9s3l122B9ysZMmhhfbGN6vRenf+5ivFBjt
# pF72iZRF8FUESt3/J90GSkD2tLzx5A+ZArv9XQ4uKMG+O18aP5cQhLwWPtijnGMd
# ZstcX9o+8w8KCTUi29vAPwD55g1dZ9H9oB4DK9lA977Mh2ZUgKajuPUZYtXSJrGY
# Ju6ay0SnRVqBlRUa9VEwggS+MIIDpqADAgECAhBZCdtAx6nLtNAzYGqLduLxMA0G
# CSqGSIb3DQEBCwUAMH8xCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBD
# b3Jwb3JhdGlvbjEfMB0GA1UECxMWU3ltYW50ZWMgVHJ1c3QgTmV0d29yazEwMC4G
# A1UEAxMnU3ltYW50ZWMgQ2xhc3MgMyBTSEEyNTYgQ29kZSBTaWduaW5nIENBMB4X
# DTE3MTIwMTAwMDAwMFoXDTIwMTIwMTIzNTk1OVowVjELMAkGA1UEBhMCREUxDzAN
# BgNVBAgMBkJheWVybjEUMBIGA1UEBwwLSG9semtpcmNoZW4xDzANBgNVBAoMBlNF
# UCBBRzEPMA0GA1UEAwwGU0VQIEFHMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEAmctykWlJhUohOQgg1EZR28U8bK05rFRu89lFmp9JbCLMVF1EbMuk+sKS
# 9kS84vu1ubCbVUaok3f77kdn3RnLomMdKvCnRZ65KO62tBjgqqwGKgzuVaJ6wVpP
# hrQmpFSJDsU0h+S6ZhSKGfWtzrGXNMXDo4Lk2Q+2z9ZX1qY5N5kx/3W2qJHeDt/T
# gTL7Q0n4Y7e7TRDFYOoUjWnfJSjAqaI7F+NuzgoFmvK2YlY2J9c77sCbVY7Zm4If
# +cp0uO20QjNpkl9UbcGDonq0E0FwSGjkHi207bOE9fh/qJUfe0G6T9h70m324GR0
# TYTIxLxUaKMJEJLCCtoQy0fbJYGPlwIDAQABo4IBXTCCAVkwCQYDVR0TBAIwADAO
# BgNVHQ8BAf8EBAMCB4AwKwYDVR0fBCQwIjAgoB6gHIYaaHR0cDovL3N2LnN5bWNi
# LmNvbS9zdi5jcmwwYQYDVR0gBFowWDBWBgZngQwBBAEwTDAjBggrBgEFBQcCARYX
# aHR0cHM6Ly9kLnN5bWNiLmNvbS9jcHMwJQYIKwYBBQUHAgIwGQwXaHR0cHM6Ly9k
# LnN5bWNiLmNvbS9ycGEwEwYDVR0lBAwwCgYIKwYBBQUHAwMwVwYIKwYBBQUHAQEE
# SzBJMB8GCCsGAQUFBzABhhNodHRwOi8vc3Yuc3ltY2QuY29tMCYGCCsGAQUFBzAC
# hhpodHRwOi8vc3Yuc3ltY2IuY29tL3N2LmNydDAfBgNVHSMEGDAWgBSWO1PweTOX
# r32D7y4rzMq3hh5yZjAdBgNVHQ4EFgQUTlTCnYm9xLWJFk8+WQNKWMlzU/4wDQYJ
# KoZIhvcNAQELBQADggEBACSg+bYIj3GGqOOfPuHuteQu48s+2Xtk5NTUbzP14zMW
# LzRv32iE3HOsaD5tDPYzW+8LftfnaNN+h2mvl3SlNbOOStJTjhebE2hlMuMqZmav
# wJnJModfgHUFp05tIEd3t/qxgLPYPR4wG+MClqr4Msnz0y2v9cko/gVzvHu3rjMC
# sYQ+VzwgSV57KDB5PZ/8R3CXzLYoFQG1MNLeFbQazWbyx8szCULeBp7r90nZNa7y
# ejlZWLfAnRpS1NoRsa/0/ENp2AeQD2Z5uxRf+PPZObxi7ckESbiOL1YcGeQWyZEJ
# FzxSEXH/HwTh/s0scgToHRoWvyr9stA3McBwjXT+KOAwggTmMIIDzqADAgECAhBi
# XE2QjNVC+6supXM/8VQZMA0GCSqGSIb3DQEBBQUAMIGVMQswCQYDVQQGEwJVUzEL
# MAkGA1UECBMCVVQxFzAVBgNVBAcTDlNhbHQgTGFrZSBDaXR5MR4wHAYDVQQKExVU
# aGUgVVNFUlRSVVNUIE5ldHdvcmsxITAfBgNVBAsTGGh0dHA6Ly93d3cudXNlcnRy
# dXN0LmNvbTEdMBsGA1UEAxMUVVROLVVTRVJGaXJzdC1PYmplY3QwHhcNMTEwNDI3
# MDAwMDAwWhcNMjAwNTMwMTA0ODM4WjB6MQswCQYDVQQGEwJHQjEbMBkGA1UECBMS
# R3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRowGAYDVQQKExFD
# T01PRE8gQ0EgTGltaXRlZDEgMB4GA1UEAxMXQ09NT0RPIFRpbWUgU3RhbXBpbmcg
# Q0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCqgvGEqVvYcbXSXSvt
# 9BMgDPmb6dGPdF5u7uspSNjIvizrCmFgzL2SjXzddLsKnmhOqnUkcyeuN/MagqVt
# uMgJRkx+oYPp4gNgpCEQJ0CaWeFtrz6CryFpWW1jzM6x9haaeYOXOh0Mr8l90U7Y
# w0ahpZiqYM5V1BIR8zsLbMaIupUu76BGRTl8rOnjrehXl1/++8IJjf6OmqU/WUb8
# xy1dhIfwb1gmw/BC/FXeZb5nOGOzEbGhJe2pm75I30x3wKoZC7b9So8seVWx/lla
# Wm1VixxD9rFVcimJTUA/vn9JAV08m1wI+8ridRUFk50IYv+6Dduq+LW/EDLKcuoI
# Js0ZAgMBAAGjggFKMIIBRjAfBgNVHSMEGDAWgBTa7WR0FJwUPKvdmam9WyhNizzJ
# 2DAdBgNVHQ4EFgQUZCKGtkqJyQQP0ARYkiuzbj0eJ2wwDgYDVR0PAQH/BAQDAgEG
# MBIGA1UdEwEB/wQIMAYBAf8CAQAwEwYDVR0lBAwwCgYIKwYBBQUHAwgwEQYDVR0g
# BAowCDAGBgRVHSAAMEIGA1UdHwQ7MDkwN6A1oDOGMWh0dHA6Ly9jcmwudXNlcnRy
# dXN0LmNvbS9VVE4tVVNFUkZpcnN0LU9iamVjdC5jcmwwdAYIKwYBBQUHAQEEaDBm
# MD0GCCsGAQUFBzAChjFodHRwOi8vY3J0LnVzZXJ0cnVzdC5jb20vVVROQWRkVHJ1
# c3RPYmplY3RfQ0EuY3J0MCUGCCsGAQUFBzABhhlodHRwOi8vb2NzcC51c2VydHJ1
# c3QuY29tMA0GCSqGSIb3DQEBBQUAA4IBAQARyT3hBeg7ZazJdDEDt9qDOMaSuv3N
# +Ntjm30ekKSYyNlYaDS18AshU55ZRv1jhd/+R6pw5D9eCJUoXxTx/SKucOS38bC2
# Vp+xZ7hog16oYNuYOfbcSV4Tp5BnS+Nu5+vwQ8fQL33/llqnA9abVKAj06XCoI75
# T9GyBiH+IV0njKCv2bBS7vzI7bec8ckmONalMu1Il5RePeA9NbSwyVivx1j/YnQW
# kmRB2sqo64sDvcFOrh+RMrjhJDt77RRoCYaWKMk7yWwowiVp9UphreAn+FOndRWw
# UTGw8UH/PlomHmB+4uNqOZrE6u4/5rITP1UDBE0LkHLU6/u8h5BRsjgZMIIE/jCC
# A+agAwIBAgIQK3PbdGMRTFpbMkryMFdySTANBgkqhkiG9w0BAQUFADB6MQswCQYD
# VQQGEwJHQjEbMBkGA1UECBMSR3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdT
# YWxmb3JkMRowGAYDVQQKExFDT01PRE8gQ0EgTGltaXRlZDEgMB4GA1UEAxMXQ09N
# T0RPIFRpbWUgU3RhbXBpbmcgQ0EwHhcNMTkwNTAyMDAwMDAwWhcNMjAwNTMwMTA0
# ODM4WjCBgzELMAkGA1UEBhMCR0IxGzAZBgNVBAgMEkdyZWF0ZXIgTWFuY2hlc3Rl
# cjEQMA4GA1UEBwwHU2FsZm9yZDEYMBYGA1UECgwPU2VjdGlnbyBMaW1pdGVkMSsw
# KQYDVQQDDCJTZWN0aWdvIFNIQS0xIFRpbWUgU3RhbXBpbmcgU2lnbmVyMIIBIjAN
# BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAv1I2gjrcdDcNeNV/FlAZZu26GpnR
# YziaDGayQNungFC/aS42LwpnP0ChSopjNZvQGcx0qhcZkSu1VSAZ+8AaOm3KOZuC
# 8rqVoRrYNMe4iXtwiHBRZmnsd/7GlHJ6zyWB7TSCmt8IFTcxtG2uHL8Y1Q3P/rXh
# xPuxR3Hp+u5jkezx7M5ZBBF8rgtgU+oq874vAg/QTF0xEy8eaQ+Fm0WWwo0Si2eu
# H69pqwaWgQDfkXyVHOaeGWTfdshgRC9J449/YGpFORNEIaW6+5H6QUDtTQK0S3/f
# 4uA9uKrzGthBg49/M+1BBuJ9nj9ThI0o2t12xr33jh44zcDLYCQD3npMqwIDAQAB
# o4IBdDCCAXAwHwYDVR0jBBgwFoAUZCKGtkqJyQQP0ARYkiuzbj0eJ2wwHQYDVR0O
# BBYEFK7u2WC6XvUsARL9jo2yVXI1Rm/xMA4GA1UdDwEB/wQEAwIGwDAMBgNVHRMB
# Af8EAjAAMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMIMEAGA1UdIAQ5MDcwNQYMKwYB
# BAGyMQECAQMIMCUwIwYIKwYBBQUHAgEWF2h0dHBzOi8vc2VjdGlnby5jb20vQ1BT
# MEIGA1UdHwQ7MDkwN6A1oDOGMWh0dHA6Ly9jcmwuc2VjdGlnby5jb20vQ09NT0RP
# VGltZVN0YW1waW5nQ0FfMi5jcmwwcgYIKwYBBQUHAQEEZjBkMD0GCCsGAQUFBzAC
# hjFodHRwOi8vY3J0LnNlY3RpZ28uY29tL0NPTU9ET1RpbWVTdGFtcGluZ0NBXzIu
# Y3J0MCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTANBgkqhkiG
# 9w0BAQUFAAOCAQEAen+pStKwpBwdDZ0tXMauWt2PRR3wnlyQ9l6scP7T2c3kGaQK
# Q3VgaoOkw5mEIDG61v5MzxP4EPdUCX7q3NIuedcHTFS3tcmdsvDyHiQU0JzHyGeq
# C2K3tPEG5OfkIUsZMpk0uRlhdwozkGdswIhKkvWhQwHzrqJvyZW9ljj3g/etfCgf
# 8zjfjiHIcWhTLcuuquIwF4MiKRi14YyJ6274fji7kE+5Xwc0EmuX1eY7kb4AFyFu
# 4m38UnnvgSW6zxPQ+90rzYG2V4lO8N3zC0o0yoX/CLmWX+sRE+DhxQOtVxzhXZIG
# vhvIPD+lIJ9p0GnBxcLJPufFcvfqG5bilK+GLjCCBVkwggRBoAMCAQICED141/l2
# SWCyYX308B7KhiowDQYJKoZIhvcNAQELBQAwgcoxCzAJBgNVBAYTAlVTMRcwFQYD
# VQQKEw5WZXJpU2lnbiwgSW5jLjEfMB0GA1UECxMWVmVyaVNpZ24gVHJ1c3QgTmV0
# d29yazE6MDgGA1UECxMxKGMpIDIwMDYgVmVyaVNpZ24sIEluYy4gLSBGb3IgYXV0
# aG9yaXplZCB1c2Ugb25seTFFMEMGA1UEAxM8VmVyaVNpZ24gQ2xhc3MgMyBQdWJs
# aWMgUHJpbWFyeSBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eSAtIEc1MB4XDTEzMTIx
# MDAwMDAwMFoXDTIzMTIwOTIzNTk1OVowfzELMAkGA1UEBhMCVVMxHTAbBgNVBAoT
# FFN5bWFudGVjIENvcnBvcmF0aW9uMR8wHQYDVQQLExZTeW1hbnRlYyBUcnVzdCBO
# ZXR3b3JrMTAwLgYDVQQDEydTeW1hbnRlYyBDbGFzcyAzIFNIQTI1NiBDb2RlIFNp
# Z25pbmcgQ0EwggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQCXgx4AFq8s
# sdIIxNdok1FgHnH24ke021hNI2JqtL9aG1H3ow0Yd2i72DarLyFQ2p7z518nTgvC
# l8gJcJOp2lwNTqQNkaC07BTOkXJULs6j20TpUhs/QTzKSuSqwOg5q1PMIdDMz3+b
# 5sLMWGqCFe49Ns8cxZcHJI7xe74xLT1u3LWZQp9LYZVfHHDuF33bi+VhiXjHaBuv
# EXgamK7EVUdT2bMy1qEORkDFl5KK0VOnmVuFNVfT6pNiYSAKxzB3JBFNYoO2unto
# gjHuZcrf+dWNsjXcjCtvanJcYISc8gyUXsBWUgBIzNP4pX3eL9cT5DiohNVGuBOG
# whud6lo43ZvbAgMBAAGjggGDMIIBfzAvBggrBgEFBQcBAQQjMCEwHwYIKwYBBQUH
# MAGGE2h0dHA6Ly9zMi5zeW1jYi5jb20wEgYDVR0TAQH/BAgwBgEB/wIBADBsBgNV
# HSAEZTBjMGEGC2CGSAGG+EUBBxcDMFIwJgYIKwYBBQUHAgEWGmh0dHA6Ly93d3cu
# c3ltYXV0aC5jb20vY3BzMCgGCCsGAQUFBwICMBwaGmh0dHA6Ly93d3cuc3ltYXV0
# aC5jb20vcnBhMDAGA1UdHwQpMCcwJaAjoCGGH2h0dHA6Ly9zMS5zeW1jYi5jb20v
# cGNhMy1nNS5jcmwwHQYDVR0lBBYwFAYIKwYBBQUHAwIGCCsGAQUFBwMDMA4GA1Ud
# DwEB/wQEAwIBBjApBgNVHREEIjAgpB4wHDEaMBgGA1UEAxMRU3ltYW50ZWNQS0kt
# MS01NjcwHQYDVR0OBBYEFJY7U/B5M5evfYPvLivMyreGHnJmMB8GA1UdIwQYMBaA
# FH/TZafC3ey78DAJ80M5+gKvMzEzMA0GCSqGSIb3DQEBCwUAA4IBAQAThRoeaak3
# 96C9pK9+HWFT/p2MXgymdR54FyPd/ewaA1U5+3GVx2Vap44w0kRaYdtwb9ohBcIu
# c7pJ8dGT/l3JzV4D4ImeP3Qe1/c4i6nWz7s1LzNYqJJW0chNO4LmeYQW/CiwsUfz
# HaI+7ofZpn+kVqU/rYQuKd58vKiqoz0EAeq6k6IOUCIpF0yH5DoRX9akJYmbBWsv
# tMkBTCd7C6wZBSKgYBU/2sn7TUyP+3Jnd/0nlMe6NQ6ISf6N/SivShK9DbOXBd5E
# DBX6NisD3MFQAfGhEV0U5eK9J0tUviuEXg+mw3QFCu+Xw4kisR93873NQ9TxTKk/
# tYuEr2Ty0BQhMYIEWTCCBFUCAQEwgZMwfzELMAkGA1UEBhMCVVMxHTAbBgNVBAoT
# FFN5bWFudGVjIENvcnBvcmF0aW9uMR8wHQYDVQQLExZTeW1hbnRlYyBUcnVzdCBO
# ZXR3b3JrMTAwLgYDVQQDEydTeW1hbnRlYyBDbGFzcyAzIFNIQTI1NiBDb2RlIFNp
# Z25pbmcgQ0ECEFkJ20DHqcu00DNgaot24vEwCQYFKw4DAhoFAKBwMBAGCisGAQQB
# gjcCAQwxAjAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcC
# AQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBRBrruJk6ZyOyqw613s
# n86fymOr0zANBgkqhkiG9w0BAQEFAASCAQAzbcgZWZNHqE6UqpAkJ6Om/4QnVSM0
# DgCyF0RtnIyyLP9F+AYd4I1YGYfTfUQFTw2SFWlSjJBrcTuo9PtTvEdWVX4c+qyq
# CekQKton9lre8L+NCTy8HUOTqnIU+tXCo/+2BiEl0VmNeQgNhUGi1Q6KuWmpqhdS
# zGJK2BHQqaIUvLXCv8YRRFcHU4Lof8EcfEYsJhPhyczokYzTCe5lw2fM052Dljze
# cslQhwCROLt+5Tl1DJHcE0uSYzQs9bfFTco5l8Cg9KHciKkvBl04pbvpBuYLERJJ
# YM1ndgdZoRv4jTEhLp6zWxa9NjFU9vlLzeGb/F3Eq1Ecr/cOFs4PrKy5oYICKDCC
# AiQGCSqGSIb3DQEJBjGCAhUwggIRAgEBMIGOMHoxCzAJBgNVBAYTAkdCMRswGQYD
# VQQIExJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcTB1NhbGZvcmQxGjAYBgNV
# BAoTEUNPTU9ETyBDQSBMaW1pdGVkMSAwHgYDVQQDExdDT01PRE8gVGltZSBTdGFt
# cGluZyBDQQIQK3PbdGMRTFpbMkryMFdySTAJBgUrDgMCGgUAoF0wGAYJKoZIhvcN
# AQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUxDxcNMTkwOTA0MTQzNTAxWjAj
# BgkqhkiG9w0BCQQxFgQU1arnMQE5Z2mZS6xqwfmc2GC4rjswDQYJKoZIhvcNAQEB
# BQAEggEAG/4pVMYFLB2Zp5Ez0CdY2aT2OPHiIEv5QVuoPp82tAbcvFWmzOht/H8p
# cC5uOa5W+WqGrcElUJ0gJ1qCBIhxOkNHIvhdDJJ50B6LOABVyXav8qcb9ZRKSNsK
# Vg57wgsPz7yJvOviuuX9ijU5OZZP5JJWa0JgeXVyN/2OSmJqIPKxCCV+oRArgwva
# 8NAnLDjrIYoHHSOo3MLYhBXYKW/gDSoe+CO7ZV5vVWcmjuMzDpCvkIkSbNU6elUZ
# 15pM7zOLA1SLxDv/m5Gg8RPolo2pQeff7FKjR5HjYscDqIuwVEm+vuYS8hS7NsYp
# 7zWUuqpxxvPQXCbHvEh2OJginA/RCA==
# SIG # End signature block
